//
// © Copyright 2020 David Vines
//
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer
//    in the documentation and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived 
//    from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
// COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
/* globals BellCode */
/* jshint esnext: true */

function Bell(scheduler, name, audio, element, clock, view) {
    this._scheduler = scheduler;
    this._name = name;
    this._clock = clock;
    this._view = view;
    this._audio = audio;
    this._inGap = false;
    this._bellCode = null;
    this._startOfGapTime = 0;
    this._callWhenBellCodeComplete = null;
    this._element = element;
}

Bell.prototype = {
    shortGapMax: 1,
    shortGap: 1,
    longGapMax: 2,
    longGap: 1,
    getName: function() {
        return this._name;
    },
    markRinging: function() {
        this._element.classList.add("ringing");
    },
    unmarkRinging: function() {
        this._element.classList.remove("ringing");
    },
    _playAudio: function() {
        this._audio.pause();
        this._audio.currentTime = 0;
        this._audio.play();
    },
    scheduleBellCode : function(bellCode, offset) {
        var when = offset;
        const bc = bellCode.toString();
        const zero = '0'.charCodeAt(0);
        for(var j=0; j< bc.length; j++) {
            if (bc.charAt(j) === "-") {
                when += this.longGap;
            } else {
                for(var k=zero; k<bc.charAt(j).charCodeAt(0); k++) {
                    this._scheduler.addRealTimeEvent(when,this.markRinging,this);
                    this._scheduler.addRealTimeEvent(when,this._playAudio,this);
                    this._scheduler.addRealTimeEvent(when+0.5,this.unmarkRinging,this);
                    when += this.shortGap;
                }
            }
        }
        this._scheduler.addRealTimeEvent(when+this.longGapMax-this.shortGap,this._view.addBellPush,this._view,
                this._clock.getClockAsStringAfterXMilliseconds(when),"from "+this._name,bellCode);
        return when-offset;
    },
    bellPushed : function(toCallWhenComplete) {
        const sizeOfGap = (Date.now() - this._startOfGapTime) / 1000;
        if (this._inGap) {
            if (sizeOfGap < this.shortGapMax) {
                this._bellCode.addShortGap();
            } else {
                this._bellCode.addLongGap();
            }
        } else {
            this._bellCode = new BellCode();
            this._inGap = true;
            this._callWhenBellCodeComplete = toCallWhenComplete;
        }
        this._startOfGapTime = Date.now();
        this._scheduler.addRealTimeEvent(this.longGapMax+0.001,this.checkEndOfBellCode,this);
    },
    checkEndOfBellCode: function() {
        const sizeOfGap = (Date.now() - this._startOfGapTime) / 1000;
        if (sizeOfGap > this.longGapMax) {
            this._inGap = false;
            this._callWhenBellCodeComplete();
        }
        // else there's been another bellPush and hence another call to this function has been scheduled and we'll check again then!
    },
    isInBellCode: function() {
        return this._inGap;
    },
    getBellCode : function() {
        return this._bellCode;
    },
    echoBellCode: function() {
        return this.scheduleBellCode(this._bellCode,1);
    },
    setTooltip: function(tip) {
        this._element.title = tip;
    }
};