//
// © Copyright 2020 David Vines
//
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer
//    in the documentation and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived 
//    from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
// COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
function BellCode(s,description) {
    if (s && s.length > 0) {
        this._code = s;
    } else if (s === ""){
        this._code = "";
    } else {
        this._code = "1";
    }
    this._description = description;
    
    if (this._description) {
        this._frozen = true;
        BellCode._knownCodes.push(this);
    } else {
        this._frozen = false;
    }
}

BellCode._knownCodes = [];

BellCode.prototype = {
    _checkNotFrozen : function() {
        if (this._frozen) {
            throw new Error("Attempting to change a constant bell code");
        }
    },
    addShortGap: function() {
        this._checkNotFrozen();
        this._code = this._code.slice(0,-1) + String.fromCharCode(this._code[this._code.length-1].charCodeAt(0)+1);
    },
    addLongGap: function() {
        this._checkNotFrozen();
        this._code = this._code + "-1";
    },
    toString: function() {
        return this._code;
    },
    freeze: function() {
        this._frozen = true;
    },
    getDescription: function() {
        if (this._description) {
            return this._description;
        } else {
            for(var i=0; i<BellCode._knownCodes.length; i++) {
                if (this._code === BellCode._knownCodes[i]._code) {
                    return BellCode._knownCodes[i]._description;
                }
            }
            return this._code;
        }
    },
    equals: function(other) {
        if (other instanceof BellCode) {
            return this._code === other._code;
        } else {
            return false;
        }
    }
};

BellCode.Empty               = new BellCode("","[empty bell code]");
BellCode.Attn                = new BellCode("1","Attention");
BellCode.EnteringSection     = new BellCode("2","Train Entering Section");
BellCode.OutOfSection        = new BellCode("2-1","Train Leaving Section");
BellCode.OfferExpress        = new BellCode("4","Express Train");
BellCode.OfferPassenger      = new BellCode("3-1","Passenger Train");
BellCode.OfferEmptyTrain     = new BellCode("2-2-1","Empty Coaching Stock");
BellCode.OfferExpressFreight = new BellCode("5","Express Freight Train");
BellCode.OfferFreight        = new BellCode("4-1","Freight Train");
BellCode.OfferSlowFreight    = new BellCode("3-2","Slow Freight");
BellCode.OfferLightLoco      = new BellCode("2-3","Light Loco");
BellCode.Shunt               = new BellCode("3-3-2","Train Entering Shunt");
BellCode.EndShunt            = new BellCode("8","Train Leaving Shunt");
BellCode.Cancel              = new BellCode("3-5","Cancel");
BellCode.WronglyDescribed    = new BellCode("5-3","Train Wrongly Described");
BellCode.Unknown             = new BellCode("9-9-9","Unknown");
BellCode.Testing             = new BellCode("@","Testing");
BellCode.OpeningBox          = new BellCode("5-5-5","Opening Box");
BellCode.ClosingBox          = new BellCode("7-5-5","Closing Box");