//
// © Copyright 2020 David Vines
//
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer
//    in the documentation and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived 
//    from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
// COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
/* globals TrainFactory, BellCode, Subclass */
/* jshint esnext: true */

function Trains(scheduler,model,kimbridgeBox,eastleighBox,redbridgeBox) {
    var t;
    t = new TrainFactory(scheduler,model,"Salisbury to Southampton Train");
    t.setActions([t.startTrain(kimbridgeBox,BellCode.OfferPassenger), t.pause(60),
                  t.updateLocation("A"),  t.pause(10),
                  t.waitAtSignal(20),     t.pause(5),
                  t.updateLocation("B"),  t.pause(10),
                  t.arrived(),
                  t.updateLocation("P1"), t.pause(120),
                  t.waitAtSignal(16),     t.depart(), t.pause(15),
                  t.updateLocation("J"),  t.pause(25),
                  t.updateLocation("D"),
                  t.waitAtSignal(13),     t.pause(30),
                  t.updateLocation(null), t.pause(150),
                  t.stopTrain(redbridgeBox)
                ]);
    this.SAL_TO_SOU = t;
    t = new TrainFactory(scheduler,model,"Southampton to Salisbury Train");
    t.setActions([t.startTrain(redbridgeBox,BellCode.OfferPassenger), t.pause(10),
                  t.waitAtSignal(5),      t.pause(5),
                  t.updateLocation("H"),  t.pause(60),
                  t.updateLocation("K"),  t.pause(20),
                  t.arrived(),
                  t.updateLocation("P2"), t.pause(120),
                  t.waitAtSignal(3),      t.depart(), t.pause(10),
                  t.updateLocation(null), t.pause(150),
                  t.stopTrain(kimbridgeBox)
                ]);
    this.SOU_TO_SAL = t;
    t = new TrainFactory(scheduler,model,"Salisbury to Eastleigh Freight");
    t.setActions([t.startTrain(kimbridgeBox,BellCode.OfferExpressFreight), t.pause(10),
                  t.updateLocation("A"),  t.pause(10),
                  t.waitAtSignal(20),     t.pause(5),
                  t.updateLocation("B"),  t.pause(10),
                  t.arrived(),
                  t.updateLocation("P1"), t.pause(30),
                  t.waitAtSignal(18),     t.depart(), t.pause(15),
                  t.updateLocation("C"),  t.pause(20),
                  t.updateLocation("UZ"), t.pause(20),
                  t.waitAtSignal(17),     t.pause(15),
                  t.updateLocation("PG"), t.pause(60),
                  t.updateLocation("PF"), t.pause(45),
                  t.updateLocation("PE"), t.pause(45),
                  t.updateLocation("PD"), t.pause(30),
                  t.setEastleighSignals(true,false), t.pause(40),
                  t.updateLocation("PC"), t.pause(10),
                  t.setEastleighSignals(true,true), t.pause(15),
                  t.updateLocation(null), t.pause(10),
                  t.setEastleighSignals(false,false),
                  t.stopTrain(eastleighBox)
                 ]);
    this.SAL_TO_ESL = t;
    t = new TrainFactory(scheduler,model,"Eastleigh to Salisbury Freight");
    t.setActions([t.setEastleighPoint(true),
                  t.startTrain(eastleighBox,BellCode.OfferExpressFreight), t.pause(10),
                  t.updateLocation("PC"),     t.pause(20),
                  t.setEastleighPoint(false), t.pause(20),
                  t.updateLocation("PD"),     t.pause(45),
                  t.updateLocation("PE"),     t.pause(45),
                  t.updateLocation("PF"),     t.pause(45),
                  t.waitAtSignal(1),          t.pause(10),
                  t.updateLocation("PH"),     t.pause(45),
                  t.waitAtSignal(2),          t.pause(10),
                  t.updateLocation("J"),      t.pause(20),
                  t.updateLocation("K"),      t.pause(20),
                  t.arrived(),
                  t.updateLocation("P2"),     t.pause(20),
                  t.waitAtSignal(3),          t.depart(), t.pause(10),
                  t.updateLocation(null),     t.pause(150),
                  t.stopTrain(kimbridgeBox)
                ]);
    this.ESL_TO_SAL = t;
    t = new TrainFactory(scheduler,model,"Eastleigh to Eastleigh Passenger");
    t.setActions([t.setEastleighPoint(true),
                  t.startTrain(eastleighBox,BellCode.OfferPassenger), t.pause(10),
                  t.updateLocation("PC"),            t.pause(20),
                  t.setEastleighPoint(false),        t.pause(20),
                  t.updateLocation("PD"),            t.pause(45),
                  t.updateLocation("PE"),            t.pause(45),
                  t.updateLocation("PF"),            t.pause(45),
                  t.waitAtSignal(1),                 t.pause(10),
                  t.updateLocation("PH"),            t.pause(45),
                  t.waitAtSignal(2),                 t.pause(10),
                  t.updateLocation("J"),             t.pause(20),
                  t.updateLocation("K"),             t.pause(20),
                  t.updateLocation("L"),             t.pause(10),
                  t.arrived(),
                  t.updateLocation("P2"),            t.pause(300),
                  t.depart(), 
                  t.updateLocation("L"),             t.pause(10),
                  t.waitAtSignal(22),                t.pause(20),
                  t.updateLocation("C"),             t.pause(20),
                  t.updateLocation("UZ"),            t.pause(20),
                  t.waitAtSignal(17),                t.pause(15),
                  t.updateLocation("PG"),            t.pause(60),
                  t.updateLocation("PF"),            t.pause(45),
                  t.updateLocation("PE"),            t.pause(45),
                  t.updateLocation("PD"),            t.pause(90),
                  t.setEastleighSignals(true,false), t.pause(40),
                  t.updateLocation("PC"),            t.pause(10),
                  t.setEastleighSignals(true,true),  t.pause(15),
                  t.updateLocation(null),            t.pause(10),
                  t.setEastleighSignals(false,false),
                  t.stopTrain(eastleighBox)
                ]);
    this.ESL_TO_ESL = t;
    t = new TrainFactory(scheduler,model,"Salisbury to Southampton Freight");
    t.setActions([t.startTrain(kimbridgeBox,BellCode.OfferExpressFreight), t.pause(10),
                  t.updateLocation("A"),  t.pause(10),
                  t.waitAtSignal(20),     t.pause(5),
                  t.updateLocation("B"),  t.pause(10),
                  t.arrived(),
                  t.updateLocation("P1"), t.pause(20),
                  t.waitAtSignal(16),     t.depart(), t.pause(15),
                  t.updateLocation("J"),  t.pause(25),
                  t.updateLocation("D"),
                  t.waitAtSignal(13),     t.pause(30),
                  t.updateLocation(null), t.pause(150),
                  t.stopTrain(redbridgeBox)
                ]);
    this.SAL_TO_SOU_freight = t;
    t = new TrainFactory(scheduler,model,"Southampton to Salisbury Freight");
    t.setActions([t.startTrain(redbridgeBox,BellCode.OfferExpressFreight), t.pause(10),
                  t.waitAtSignal(5),      t.pause(5),
                  t.updateLocation("H"),  t.pause(60),
                  t.updateLocation("K"),  t.pause(20),
                  t.arrived(),
                  t.updateLocation("P2"), t.pause(20),
                  t.waitAtSignal(3),      t.depart(), t.pause(10),
                  t.updateLocation(null), t.pause(150),
                  t.stopTrain(kimbridgeBox)
                ]);
    this.SOU_TO_SAL_freight = t;
}

function Timetable(scheduler,name) {
    this._scheduler = scheduler;
    this._name = name;
}
Timetable.prototype = {
    getName: function() { return this._name; },
    run: function(clock) {
        throw new Error("run not overriden");
    }
};

function SingleTrainTimetable(scheduler,factory) {
    Timetable.call(this,scheduler,factory.getName());
    this._factory = factory;
}
Subclass.make(SingleTrainTimetable,Timetable);
SingleTrainTimetable.prototype.run = function(clock) {
    const train = this._factory.make();
    this._scheduler.addRealTimeEvent(0,train.doNextStep,train);
};

function TrainRecord(offset,arrival,origin,destination,departure,factory,view) {
    this._offset = offset;
    this._arrival = arrival;
    this._origin = origin;
    this._destination = destination;
    this._departure = departure;
    this._factory = factory;
    this._row = null;
    this._view = view;
}
TrainRecord.prototype = {
    getOffset: function() { return this._offset; },
    _addTrainToTimetable: function(view) {
        this._row = view.addTrainToTimetable(this._arrival,this._origin,this._destination,this._departure);
    },
    _listener: function() {
        const us = this; // Capture this
        return {arrival: function() { us.arrival(); }, departure: function() { us.departure(); } };
    },
    prepare: function(clock,view) {
        const train = this._factory.make();
        this._addTrainToTimetable(view);
        this._clock = clock;
        train.addListener(this._listener());
        return train;
    },
    arrival: function() {
        if (this._arrival !== "pass") {
            this._view.updateActualArrivalTime(this._row,this._clock.getHh(),this._clock.getMm(),this._clock.getSs());
        }
    },
    departure: function() {
        this._view.updateActualDepartureTime(this._row,this._clock.getHh(),this._clock.getMm(),this._clock.getSs());
    }
};

function MultipleTrainTimetable(scheduler,name,view,starthh,startmm,startss) {
    Timetable.call(this,scheduler,name);
    this._view = view;
    this._starthh = starthh;
    this._startmm = startmm;
    this._startss = startss;
    this._trains = [];
}
Subclass.make(MultipleTrainTimetable,Timetable);
MultipleTrainTimetable.prototype.addTrain = function(offset,arrival,origin,destination,departure,factory) {
    this._trains.push(new TrainRecord(offset,arrival,origin,destination,departure,factory,this._view));
};
MultipleTrainTimetable.prototype.run = function(clock) {
    this._view.showTimetable();
    clock.setClock(this._starthh,this._startmm,this._startss);
    for(var i=0; i<this._trains.length; i++) {
        var train = this._trains[i].prepare(clock,this._view);
        this._scheduler.addGameTimeEvent(this._trains[i].getOffset(),train.doNextStep,train);
    }
};

function Timetables(scheduler,model,view,kimbridgeBox,eastleighBox,redbridgeBox) {
    const trains = new Trains(scheduler,model,kimbridgeBox,eastleighBox,redbridgeBox);
    const salToSou = new SingleTrainTimetable(scheduler,trains.SAL_TO_SOU);
    const souToSal = new SingleTrainTimetable(scheduler,trains.SOU_TO_SAL);
    const salToEsl = new SingleTrainTimetable(scheduler,trains.SAL_TO_ESL);
    const eslToSal = new SingleTrainTimetable(scheduler,trains.ESL_TO_SAL);
    const eslToEsl = new SingleTrainTimetable(scheduler,trains.ESL_TO_ESL);
    
    const jan242019 = new MultipleTrainTimetable(scheduler,"11:55-13:00 January 24 2019",view,11,55,0);
    jan242019.addTrain(25,"1159","Great Malvern","Southampton","1201",trains.SAL_TO_SOU);
    jan242019.addTrain(50,"1202","Romsey (circular)","Romsey (circular)","1207",trains.ESL_TO_ESL);
    jan242019.addTrain(725,"pass","Salisbury T&R.S.M.D","Bournemouth T&R.S.M.D","1210",trains.SAL_TO_SOU_freight);
    jan242019.addTrain(1185,"1218","Portsmouth Harbour","Cardiff Central","1220",trains.SOU_TO_SAL);
    jan242019.addTrain(1605,"1225","Salisbury","Romsey (circular)","1227",trains.SAL_TO_SOU);
    jan242019.addTrain(1860,"pass","Fareham A.R.C. Sidings","Whatley Quarry","1233",trains.ESL_TO_SAL);
    jan242019.addTrain(2265,"1236","Southampton","Great Malvern","1238",trains.SOU_TO_SAL);
    jan242019.addTrain(3030,"1248","Romsey (circular)","Salisbury","1250",trains.SOU_TO_SAL);
    jan242019.addTrain(3045,"1248","Cardiff Central","Portsmouth Harbour","1250",trains.SAL_TO_SOU);
    
    const apr092020 = new MultipleTrainTimetable(scheduler,"11:50-12:30 April 09 2020",view,11,50,0);
    apr092020.addTrain(0,"1153","Cardiff Central","Portsmouth Harbour","1155",trains.SAL_TO_SOU);
    apr092020.addTrain(325,"1202","Romsey (circular)","Romsey (circular)","1207",trains.ESL_TO_ESL);
    apr092020.addTrain(1080,"pass","Salisbury T&R.S.M.D","Bournemouth T&R.S.M.D","1215",trains.SAL_TO_SOU_freight);
    apr092020.addTrain(1740,"1222","Portsmouth Harbour","Cardiff Central","1223",trains.SOU_TO_SAL);
    
    this.list = [souToSal,salToSou,salToEsl,eslToSal,eslToEsl,apr092020,jan242019];
}